#import "Table.h"
#import "Cards.h"
#import "Player.h"
#import "TableView.h"
#import "Hand.h"
#import "Buttons.h"
#import "DOMTreeNode.h"
#import "iDeckAppDelegate.h"

@implementation Table


- (void) update : (NSURL*) url {
	NSLog(@"updating!");
	if (root) [root  release];
	XMLParser *parser = [[XMLParser alloc] init];
	root = [parser parseXMLFile:url];
	//[root dump];
	[root retain];    
	[parser release];
	[self display];
}

- (void) send :(NSString*) mesg {
	XMLParser *parser = [[XMLParser alloc] init];
	NSMutableString * url = [[NSMutableString alloc] init];
	[url appendFormat:@"http://dan-lynch.com/icards.php?user_id=%d&table_id=%d&message=%@:%@&action=message", [parent user_id], [parent table_id], parent->username, mesg];
	NSString* escapedUrl = [url   
		stringByAddingPercentEscapesUsingEncoding:NSUTF8StringEncoding]; 
	NSLog(@"send: %@\n", escapedUrl);
	[parser parseXMLFile:[NSURL URLWithString:escapedUrl]];
	[parser release];
}

/* queries and then displays */

- (void) queryAndViewTable {

	if (root) [root  release];
	XMLParser *parser = [[XMLParser alloc] init];
	NSMutableString * url = [[NSMutableString alloc] init];

	[url appendFormat:@"http://dan-lynch.com/icards.php?user_id=%d&table_id=%d", [parent user_id], [parent table_id]];
	root = [parser parseXMLFile:[NSURL URLWithString:url]];
	NSLog(@"queryAndViewTable: %@\n", url);
	//[root dump];
	[root retain];    
	[parser release];
	[self display];

}

- (void) leave {
	XMLParser *parser = [[XMLParser alloc] init];
	NSMutableString * url = [[NSMutableString alloc] init];
	[url appendFormat:@"http://dan-lynch.com/icards.php?user_id=%d&table_id=%d&action=leave", [parent user_id], [parent table_id]];
	NSLog(@"leave: %@\n", url);
	[parser parseXMLFile:[NSURL URLWithString:url]];
	[parser release];
}

- (void) join {

	XMLParser *parser = [[XMLParser alloc] init];
	NSMutableString * url = [[NSMutableString alloc] init];
	[url appendFormat:@"http://dan-lynch.com/icards.php?user_id=%d&table_id=%d&action=join", parent->user_id, parent->table_id];
	NSLog(@"join: %@\n", url);
	DOMTreeNode * userInfo = [parser parseXMLFile:[NSURL URLWithString:url]];
	[parent setUserID:[node(userInfo, @"user_id") intValue]];
	NSLog(@"received confirmation: %d\n", parent->user_id);
	[self queryAndViewTable];
}

- (void) swipe : (int) type {

	XMLParser *parser = [[XMLParser alloc] init];
	NSMutableString * url = [[NSMutableString alloc] init];
	[url appendFormat:@"http://dan-lynch.com/icards.php?user_id=%d&table_id=%d&swipe=%d&action=swipe", parent->user_id, parent->table_id, type];
	NSLog(@"swipe: %@\n", url);
	(void*)[parser parseXMLFile:[NSURL URLWithString:url]];
	[self queryAndViewTable];
}


- (void) dealAction {
	if (dealState) {
		[self deal];
	} else {
		[self shuffle];	
	}
	dealState = !dealState;
}

- (void) deal {
	if (root) [root  release];
	XMLParser *parser = [[XMLParser alloc] init];
	NSMutableString * url = [[NSMutableString alloc] init];
	[url appendFormat:@"http://dan-lynch.com/icards.php?user_id=%d&table_id=%d&action=deal", [parent user_id], [parent table_id]];
	root = [parser parseXMLFile:[NSURL URLWithString:url]];
	NSLog(@"deal: %@\n", url);
	[root retain];    
	[parser release];
	[self display];
}

- (void) shuffle {

	if (root) [root  release];
	XMLParser *parser = [[XMLParser alloc] init];
	NSMutableString * url = [[NSMutableString alloc] init];
	[url appendFormat:@"http://dan-lynch.com/icards.php?user_id=%d&table_id=%d&action=shuffle", [parent user_id], [parent table_id]];
	root = [parser parseXMLFile:[NSURL URLWithString:url]];
	NSLog(@"shuffle: %@\n", url);
	[root retain];    
	[parser release];
	[self display];
}

- (void) deinitializeTable {	
	[self leave];
	[parent leaveTable];

}

- (void) initializeTable {

	viewTable = true;
	zoomScale = 1.0;
	[self join];
	[self queryAndViewTable];
	//    self.title = node(root, @"name");
	text = [[UITextView alloc] initWithFrame:CGRectMake(50,0,235,40)];
	NSString * str = [[NSString alloc] initWithFormat:@"Welcome to %@\n", node(root, @"name")];
	text.text = str;
	[str retain];
	text.font = [UIFont fontWithName:@"Courier" size:14.0];
	text.backgroundColor = [UIColor clearColor];
	text.textColor = [UIColor whiteColor];
	text.editable = NO;

	[self.view addSubview:text];

	CGRect rect = CGRectMake(0.0f, 0.0f, 40.0f, 40.0f);
	rect.origin.x = 140;
	rect.origin.y = 120;

	rect.origin.x = 280;
	rect.origin.y = 420;
	Buttons * hand = [[Buttons alloc] initWithFrame:rect];
	[hand setImage:[UIImage imageNamed:@"hand.png"]];
	[hand setEvent:VIEWHAND];
	[hand setUserInteractionEnabled:YES];
	[self.view addSubview:hand];

	rect.origin.x = 280;
	rect.origin.y = 0;
	Buttons * message = [[Buttons alloc] initWithFrame:rect];
	[message setImage:[UIImage imageNamed:@"message.png"]];
	[message setEvent:MESSAGE];
	[message setUserInteractionEnabled:YES];
	[self.view addSubview:message];

	rect.origin.x = 0;
	rect.origin.y = 420;
	Buttons * exit = [[Buttons alloc] initWithFrame:rect];
	[exit setImage:[UIImage imageNamed:@"exit.png"]];
	[exit setEvent:LEAVE];
	[exit setUserInteractionEnabled:YES];
	[self.view addSubview:exit];

	self.title = node(root, @"name");
}

- (Player *) updatePlayer : (CGPoint) p owner:(int)owner {

	Player * playerInstance = NULL;

	for(unsigned int i=0; i<[[[self view] subviews] count]; i++) {

		if ([[[[self view] subviews] objectAtIndex:i] isKindOfClass:[Player class]]) {
			Player * somePlayer = [[[self view] subviews] objectAtIndex:i];
			if ([somePlayer owner] == owner) {
				playerInstance = somePlayer;
				[playerInstance setMove:p];
				[playerInstance setOwner:owner];
				break;
			}
		}
	}

	if (!playerInstance) {
		CGRect dragRect = CGRectMake(0.0f, 0.0f, 64.0f, 64.0f);
		dragRect.origin = p;
		playerInstance = [[Player alloc] initWithFrame:dragRect];

		[playerInstance setOwner:owner];
		if ([playerInstance owner] == [parent user_id]) {
			[playerInstance setImage:[UIImage imageNamed:@"playerSelf.png"]];
		} else {
			[playerInstance setImage:[UIImage imageNamed:@"player.png"]];			
		}
		[playerInstance setUserInteractionEnabled:YES];
		[self.view addSubview:playerInstance];
		[playerInstance release];
	}
	return playerInstance;

}

- (Card *) updateCard : (CGPoint) p owner:(int)owner card_id:(int)card_id suit:(NSString *)suit num:(NSString *)number face:(NSString *) face{

	Card * cardInstance = NULL;

	for(unsigned int i=0; i<[[[self view] subviews] count]; i++) {

		if ([[[[self view] subviews] objectAtIndex:i] isKindOfClass:[Card class]]) {
			Card * someCard = [[[self view] subviews] objectAtIndex:i];
			if (someCard->card_id == card_id) {
				cardInstance = someCard;
				cardInstance->moveTo = p;
				cardInstance->user_id = owner;

				if (eq(face,@"1")) {
					cardInstance.image = [cardInstance.animationImages objectAtIndex:FACE_UP];
					cardInstance->face = true;
				} else {
					if (parent->user_id == cardInstance->user_id) cardInstance.image = [cardInstance.animationImages objectAtIndex:FACE_USERS];
					else cardInstance.image = [cardInstance.animationImages objectAtIndex:FACE_DOWN];
					cardInstance->face = false;
				}

				break;
			}

		}

	}

	if (!cardInstance) {
		CGRect dragRect = CGRectMake(0.0f, 0.0f, 48.0f, 64.0f);
		dragRect.origin = p;
		dragRect.origin.x;
		dragRect.origin.y;

		cardInstance = [[Card alloc] initWithFrame:dragRect];
		[cardInstance setDirty:false];

		NSMutableString * c = [[NSMutableString alloc] init];
		NSMutableString * use = [[NSMutableString alloc] init];
		[c appendFormat:@"%@%@.png", suit, number];
		[use appendFormat:@"%@%@-user.png", suit, number];



		[cardInstance setCard:card_id];
		[cardInstance setOwner:owner];
		[cardInstance setNum:number];
		[cardInstance setSuit:suit];
		cardInstance.animationImages = [NSArray arrayWithObjects:
			[UIImage imageNamed:c],
			[UIImage imageNamed:use],
			[UIImage imageNamed:@"back-blue.png"], 
			[UIImage imageNamed:@"back-blue-light.png"],nil];

		if (eq(face, @"1")) {            
			cardInstance.image = [cardInstance.animationImages objectAtIndex:FACE_UP];
			cardInstance->face = true;
		} else if (parent->user_id == owner) {
			cardInstance.image = [cardInstance.animationImages objectAtIndex:FACE_USERS];
			cardInstance->face = false;
		} else {
			cardInstance.image = [cardInstance.animationImages objectAtIndex:FACE_DOWN];
			cardInstance->face = false;
		}

		[cardInstance setUserInteractionEnabled:YES];
		[self.view addSubview:cardInstance];
		[cardInstance release];
	}
	return cardInstance;

}


- (void) testBounds{
	CGPoint p1, p4;
	p1.x = 0;p1.y = 0;
	p4.x = 320-48;p4.y = 480-64-16;

	[self updateCard:p1 owner:1 card_id:1 suit:@"c" num:@"4" face:@"1"];
	[self updateCard:p4 owner:1 card_id:1 suit:@"c" num:@"4" face:@"1"];

}

- (void) viewHand {
	viewTable = false;	
	//[[self view] addSubview:(UIView*)[parent hand]];
	//[[self view] bringSubviewToFront:(UIView*)[parent hand]];
	//[[parent hand] showHand];
}

- (void) viewTable {
	viewTable = true;	
}


/* no queries, simply displays */
- (void)display{

	if ([node(root,@"deal") isEqualToString:@"true"]) {
		printf("first deal\n");
		[[parent hand] freeCards]; // lose cards in hand
		[parent freePlayers];
	}


	if ([node(root,@"dealer") intValue] == [parent user_id]) {
		[parent setDealer:true];
		NSLog(@"you are the dealer");
	} else {
		NSLog(@"the dealer is %@", node(root,@"dealer"));
	}

	/* text messages */
	NSMutableString * message = [[NSMutableString alloc] init];
	for (DOMTreeNode *mesg in objs(root, @"messages")) {
		if (is(mesg,@"message")) {
			[message appendFormat:@"%@\n", node(mesg,@"text")]; 
		}
	}   
	text.text = message;


	for (DOMTreeNode *player in objs(root, @"players")) {
		if (is(player,@"player")) {

			CGPoint playerPoint;
			playerPoint.x = [node(player, @"x") floatValue];
			playerPoint.y = [node(player, @"y") floatValue];
			[self updatePlayer:playerPoint owner:[node(player, @"user_id") intValue]];


			for(DOMTreeNode *card in objs(player, @"hand")) {
				if (is(card, @"card")) {

					CGPoint p1;
					p1.x = [node(card, @"x") floatValue];
					p1.y = [node(card, @"y") floatValue];

					Card * cardInstance = [self updateCard:p1 owner: [node(player,@"user_id") intValue] card_id:[node(card,@"card_id") intValue] suit:node(card, @"suit") num:node(card, @"number") face:node(card, @"face")];

					if (([parent user_id] == [node(player,@"user_id") intValue])) {
						[[parent cards] setObject:cardInstance forKey:node(card,@"card_id")];
						[[[parent hand] cards] setObject:cardInstance forKey:node(card,@"card_id")];
						[cardInstance retain];
					}
				}
			}

		}
	}

	for (DOMTreeNode *card in objs(root, @"deck")) {
		if (is(card,@"card")) {

			CGPoint p1;
			p1.x = [node(card, @"x") floatValue];
			p1.y = [node(card, @"y") floatValue];

			Card * cardInstance = [self updateCard:p1 owner: 0 card_id:[node(card,@"card_id") intValue] suit:node(card, @"suit") num:node(card, @"number") face:node(card, @"face")];
			if ([parent isDealer]) {
				[[parent cards] setObject:cardInstance forKey:node(card,@"card_id")];
				[cardInstance retain];
			}
		}

	}

	// PLACE PLAYERS BEHIND ALL CARDS
	for(id x in [[self view] subviews]) {
		if ([x isKindOfClass:[Player class]]) {
			[[self view] sendSubviewToBack:(UIView*)x];
		}
	}
	[(TableView*)[self view] sendBack];



	[[self view] setNeedsDisplay];

}

-(BOOL)canBecomeFirstResponder {
	return YES;
}

-(void)viewDidAppear:(BOOL)animated {
	parent = (iDeckAppDelegate*) [[UIApplication sharedApplication] delegate];
	TableView *contentView = [[TableView alloc] initWithFrame:[[UIScreen mainScreen] applicationFrame]];
	self.view = contentView;
	[contentView release];

	[super viewDidAppear:animated];
	[self becomeFirstResponder];
}

- (void)viewWillDisappear:(BOOL)animated {
	[self resignFirstResponder];
	[super viewWillDisappear:animated];
}
- (void)motionBegan:(UIEventSubtype)motion withEvent:(UIEvent *)event {
}

- (void)motionEnded:(UIEventSubtype)motion withEvent:(UIEvent *)event {
	NSLog(@"motion ended event \n");

	if (motion == UIEventSubtypeMotionShake ) {
		NSLog(@"shakened acknowledged\n");

		[self dealAction];
	}
	else {
		NSLog(@"not shaken\n");
	}
}

@end
