#import "iDeckAppDelegate.h"
#import "HomeViewController.h"
#import "Cards.h"
#import "Player.h"
#import "Table.h"
#import	"TableView.h"
#import "Hand.h"
#import "DOMTreeNode.h"
#import "TablesViewController.h"

@implementation iDeckAppDelegate

@synthesize window, navigationController, cards, table, hand, username, tablesViewController, tableTypeToCreate;


#pragma mark -
#pragma mark Application lifecycle

- (int) user_id { return user_id; }
- (int) table_id { return table_id; }
-(bool)isDealer {return dealer;}
- (NSString *) getUsername {return username; }
- (void) setUserID:(int)uid { user_id = uid; }
- (void) setTableID:(int)tid {table_id = tid;}
- (void) setDealer:(bool)deal {dealer = deal; }
-(void) setUsername : (NSString*)uname{
	username = uname;
	[username retain];
	NSLog(@"username is %@\n", username);
}
/* polling */
- (void) checkTime {
	if (table_id == -1) return;
	time(&now);
	if (now-start >= interval) {
		// send update to server!
		printf("sending updates to server...\n");

		NSMutableString * url = [[NSMutableString alloc] init];
		[url appendFormat:@"http://dan-lynch.com/icards.php?user_id=%d&table_id=%d&action=update", user_id, table_id];

		for(unsigned int i=0; i<[[[table view] subviews] count]; i++) {

			if ([[[[table view] subviews] objectAtIndex:i] isKindOfClass:[Card class]]) {
				Card * card = [[[table view] subviews] objectAtIndex:i];

				if (card->dirty) {
					CGPoint pos = [card loc];
					int val = (card->face) ? 1 : 0;
					[url appendFormat:@"&%d=%09f,%09f,%d,%d",card->card_id,pos.x,pos.y,val,card->user_id];
					card->dirty = false;
				}
			}

		}


		NSLog(@"sending %@\n", url);

		/* update table information and get other peoples updates if any */
		[table update:[[NSURL alloc] initWithString:url]];
		time(&start); //reset timer
	}

}

- (void ) startTimer {
	timer = [NSTimer scheduledTimerWithTimeInterval:6 target: self selector:@selector(checkTime) userInfo:nil repeats: YES];
}



- (void) stopTimer {
	[timer invalidate];
	timer = nil;
}

- (void) initializeTable {
	if (!cards) cards = [[NSMutableDictionary alloc] init];
	[table initializeTable];	
	// add the background!
	[(TableView*)table.view addBackground];
}

- (void) freeCards {
	if (cards) {
		[cards dealloc];
		cards = NULL;
		cards = [[NSMutableDictionary alloc] init];
	}

	[hand freeCards];

	// remove cards, players, button, and background
	while( [[[ table view] subviews] count] > 0 ) {
		[[[[table view] subviews] objectAtIndex:0] removeFromSuperview]; 
	}

}

- (void) freePlayers {
	for(id x in [[table view] subviews]) {
		if ([x isKindOfClass:[Player class]]) {
			[x removeFromSuperview];
		}
	}
}

- (void) removePlayer:(int)uid {
	for(id x in [[table view] subviews]) {
		if ([x isKindOfClass:[Player class]]) {
			Player * p = (Player*)x;
			if (p->user_id == uid) [x removeFromSuperview];
		}
	}
}



- (void) leaveTable {
	table_id = -1;
	[self freeCards];
	[tablesViewController.listOfTables reloadData];
}

- (void) joinNetwork {

	XMLParser *parser = [[XMLParser alloc] init];
	NSMutableString * url = [[NSMutableString alloc] init];
	[url appendFormat:@"http://dan-lynch.com/icards.php?username=%@&action=join", username];
	NSLog(@"join: %@\n", url);
	DOMTreeNode * userInfo = [parser parseXMLFile:[NSURL URLWithString:url]];
	user_id = [node(userInfo, @"user_id") intValue];
	NSLog(@"received UID: %d\n", user_id);

}

-(void) setTableType : (NSString*)ttype{
	tableTypeToCreate = ttype;
	[tableTypeToCreate retain];
	NSLog(@"tableTypeToCreate is %@\n", tableTypeToCreate);
}

- (void)applicationDidFinishLaunching:(UIApplication *)application {    

	/* start timer */
	time(&start);
	interval = 2; // 2 seconds after touches... 

	cards = [[NSMutableDictionary alloc] init];
	username = [[NSString alloc] initWithString:@"Jacky"];
	user_id = -1;
	table_id = -1;
	dealer = false;
	timer = NULL;
	table = [[Table alloc] init];
	hand = [[Hand alloc] init];

	tablesViewController = [[TablesViewController alloc] initWithNibName:@"TablesViewController" bundle:[NSBundle mainBundle]];

	[window addSubview:[navigationController view]]; 

	navigationController.navigationBar.barStyle = UIBarStyleBlack;

	[window makeKeyAndVisible];

	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(shuffleMe) name:@"shake" object:nil];
}

- (void) shuffleMe {
	NSLog(@"shake it baby");
	[table dealAction];
}

- (void)applicationWillTerminate:(UIApplication *)application {
	// Save data if appropriate
	if (table_id != -1) [table leave];
}



#pragma mark -
#pragma mark Memory management

- (void)dealloc {
	[table release];
	[hand release];
	[navigationController release];
	[tablesViewController release];
	[window release];
	[super dealloc];
}


@end

